module Umbreo
	module Models
		class Blueprint

			OPERATING_SYSTEM = %w(linux windows)

			def initialize(credentials = {}, name_or_id = nil, status = false)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
				@status     = status
				@errors     = []
			end

			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					url  =  if @status
										"#{@endpoint}/api/v1/blueprints/custom"
									else
										"#{@endpoint}/api/v1/blueprints"
									end

					data = Typhoeus.get(
						          url,
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            filter:             filter
						          }
						        )

		    	@data = JSON.parse data.response_body
				end
			end

			def all(filter = {})
				@errors << "Please choose correct one of os |#{OPERATING_SYSTEM.join(", ")}|" if OPERATING_SYSTEM.exclude?(filter[:os].to_s) && filter[:os].present?
				@errors << "Search keyword is required." if filter[:keyword].blank? && filter[:search]
				
				if valid?
					retrieve_data(filter)

		    	if @data['roles'].present?
		    		page   = " | Page: #{filter[:page]} of #{@data['total_page']} pages"
		    		header =  if @status
							    			"List My Custom Blueprint#{ page if @data['total_page'] > 1 }"
							    		else
							    			"List Blueprint#{ page if @data['total_page'] > 1 }"
							    		end

	          Helpers::Table.show_table(@data['roles'], header, ['ID', 'Name', 'Slug', 'OS', 'Description'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			def show
        Helpers::ErrorException.rescue do
      		url = if @status
									"#{@endpoint}/api/v1/blueprints/show_custom"
								else
									"#{@endpoint}/api/v1/blueprints/show"
								end

					response = Typhoeus.get(
	          url,
	          body: {
	            authenticate_token: @api_key,
	            email:              @email,
	            id:                 @name_or_id
	          }
	        )

	        response = JSON.parse response.response_body rescue nil

          if response['success']
          	Helpers::AlertMessage.show_content_information_message(response['id']) { "ID: #message" }
          	Helpers::AlertMessage.show_content_information_message(response['name']) { "Name: #message" }
          	Helpers::AlertMessage.show_content_information_message(response['description']) { "Description: #message" }
          	Helpers::AlertMessage.show_content_information_message(response['os']) { "OS: #message" }
          else
          	Helpers::AlertMessage.show_error_message(response['message'])
          end
        end
			end

			def delete
				data = Typhoeus.delete(
          "#{@endpoint}/api/v1/blueprints/#{@name_or_id}",
          body: {
            authenticate_token: @api_key,
            email:              @email
          }
        )

        @data = JSON.parse data.response_body

        if @data["success"]
        	Helpers::AlertMessage.show_success_message(@data["message"])
        else
        	Helpers::AlertMessage.show_error_message(@data["message"])
        end
			end

			def create!(attributes = {})
				@create_name        = attributes[:name]
				@create_description = attributes[:description]
				@create_os          = attributes[:os]
				@create_system      = attributes[:system]
				@create_profiles    = attributes[:profiles]

				create_validation!

				if valid?
					Helpers::ErrorException.rescue do
						data = Typhoeus.post(
	              "#{@endpoint}/api/v1/blueprints",
	              body: {
	                authenticate_token:   @api_key,
	                email:                @email,
	                user_role: {           
	                  name:               @create_name,
	                  description:        @create_description,
	                  os:                 @create_os,
	                  umbreo_profile_ids: @create_profiles
	                }
	              }
	            )

						@data = JSON.parse data.response_body

						if @data["success"]
							Helpers::AlertMessage.show_success_message(@data['message'])
						else
							Helpers::AlertMessage.show_error_message(@data['message'])
						end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			def create_validation!
				@errors << "Name of blueprint is required" if @create_name.blank?
				@errors << "Operating system is required" if @create_os.blank?
				@errors << "System is required" if @create_system.blank?
				@errors << "Profiles is required" if @create_profiles.blank?
			end

			# 
			# get json files params of blueprint
			# 
			def export
				@errors << "ID or Slug is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = Typhoeus.get(
		            "#{@endpoint}/api/v1/blueprints/export",
		            body: {
		              authenticate_token:   @api_key,
		              email:                @email,
		              id:                   @name_or_id,
		              status:               @status
		            }
		          )

		        @data = JSON.parse data.response_body rescue nil
		        role = @data["user_role"] || @data["umbreo_role"]

						if role.present?
		        	name_file = role["name"].downcase.titleize.delete(" ").underscore
		        	Helpers::FileGenerator.create(name_file, role)
		          Helpers::AlertMessage.show_success_message("Success export blueprint. your blueprint is saved on json file with name #{name_file}.json")
		        else
		        	Helpers::AlertMessage.show_error_message(@data["message"])
		        end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?
				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end
				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/blueprints/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			def error
				@errors.first
			end

			def valid?
				@errors.blank?
			end

		end
	end
end